import copy
import json
import os
from functools import lru_cache
from pathlib import Path

from itsimodels.core.base_models import ConfModel, ImageModel
from itsicli.content_packs.model_formats import to_conf, to_image, image_file_extension
from itsicli.content_packs.content_types import ContentTypes
from itsicli.setup_logging import logger

CONTENT_PACK_SCREENSHOTS_DIR = ('appserver', 'static', 'screenshots')
ITSI_DIR = 'itsi'

class ContentPackConfig(object):

    file_name = 'config.json'
    file_dir = ITSI_DIR
    attr_id = 'id'
    attr_title = 'title'
    attr_description = 'description'
    attr_version = 'version'

    def __init__(self, home_path):
        self.path = home_path.joinpath(self.file_dir, self.file_name)
        self.data = {}

        self.read()

        # cp_id = self.get_cp_id()
        # if cp_id:
        #     self.verify_dir_matches_cp_id(cp_id)

    def get_cp_id(self):
        if self.data and self.data.get('id'):
            return self.data.get('id')

    def get_cp_title(self):
        if self.data and self.data.get('title'):
            return self.data.get('title')

    def verify_dir_matches_cp_id(self, cp_id):

        if not cp_id:
            msg = "\nContent pack id not provided."
            logger.error(msg)
            raise InvalidWorkspace(msg)

        curr_dir_name = os.path.basename(os.getcwd())
        if not curr_dir_name == cp_id:
            msg = '\n'.join([
                "\nContent pack id {} does not match the working directory name {}".format(
                    cp_id, curr_dir_name
                ),
                "\nMake sure you're in the correct directory, or fix the content pack id ",
                "in your config.json to match the working directory name."
            ])
            logger.error(msg)
            raise InvalidWorkspace(msg)


    def read(self):
        if not self.exists():
            return

        with open(self.path, 'r') as fobj:
            contents = fobj.read()

        self.data = json.loads(contents)

    def update(self, data):
        self.data.update(data)

    def write(self):
        data = copy.deepcopy(self.data)

        data.setdefault(self.attr_description, '')
        data.setdefault(self.attr_version, '1.0.0')

        with open(self.path, 'w') as fobj:
            fobj.write(json.dumps(data, sort_keys=True, indent=4))

    def exists(self):
        return self.path.exists()

    def __getattr__(self, name):
        if name in (
                self.attr_id,
                self.attr_title,
                self.attr_description,
                self.attr_version
        ):
            return self.data.get(name)

        logger.error('attribute {} does not exist'.format(name))
        raise self.__getattribute__(name)


class ContentPackManifest(ContentPackConfig):

    file_name = 'manifest.json'
    file_dir = ITSI_DIR
    attr_main_screenshot = 'main_screenshot'
    attr_screenshots = 'screenshots'

    def add_content_model(self, content_model):
        content_type = content_model.content_type
        content_objects = self.data.setdefault(content_type, [])

        file_name = str(content_model.model_file.file_name)
        content_objects.append(file_name)

    def write(self):
        data = copy.deepcopy(self.data)

        data.setdefault(self.attr_main_screenshot, {
            'thumb': None,
            'path': None
        })
        data.setdefault(self.attr_screenshots, [])

        for content_type in ContentTypes.keys():
            content_objects = data.get(content_type, [])
            if not content_objects:
                continue

            data[content_type] = sorted(list(set(content_objects)))

        with open(self.path, 'w') as fobj:
            fobj.write(json.dumps(data, sort_keys=True, indent=4))

    def __getattr__(self, name):
        data_keys = list(ContentTypes.keys())
        data_keys.extend([
            self.attr_main_screenshot,
            self.attr_screenshots
        ])

        if name in data_keys:
            return self.data.get(name)

        logger.error('attribute {} does not exist'.format(name))
        raise self.__getattribute__(name)


class ModelFile(object):

    file_extension = 'json'

    def __init__(self, model):
        self.model = model

    @property
    def contents(self):
        json_str = json.dumps(self.model.to_dict(), sort_keys=True, indent=4)
        return json_str.encode('utf-8')

    @property
    def file_name(self):
        return Path(
            '{model_key}.{ext}'.format(
                model_key=self.model.get_key(),
                ext=self.file_extension
            )
        )


class ModelConfFile(ModelFile):

    file_extension = 'conf'

    @property
    def contents(self):
        return to_conf(self.model).encode('utf-8')


class ModelImageFile(ModelFile):

    @property
    def contents(self):
        return to_image(self.model)

    @property
    def file_extension(self):
        return image_file_extension(self.model)


class ContentModel(object):

    model_file_types = {
        ConfModel: ModelConfFile,
        ImageModel: ModelImageFile
    }

    @classmethod
    def create_model_file(cls, model):
        model_file_class = ModelFile

        for base_model_class, file_class in cls.model_file_types.items():
            if issubclass(model.__class__, base_model_class):
                model_file_class = file_class
                break

        return model_file_class(model)

    def __init__(self, root_path, model):
        self.root_path = root_path.joinpath(ITSI_DIR)
        self.model_file = self.create_model_file(model)

    def write(self):
        os.makedirs(self.path.parent, exist_ok=True)

        with open(self.path, 'wb') as fobj:
            fobj.write(self.model_file.contents)

    @property
    @lru_cache(maxsize=1)
    def content_type(self):
        for content_type, model_class in ContentTypes.items():
            if model_class == self.model_file.model.__class__:
                return content_type
        return None

    @property
    def path(self):
        content_type_dir = self.root_path.joinpath(self.content_type)

        return content_type_dir.joinpath(self.model_file.file_name)

class InvalidWorkspace(Exception):
    """Thrown when the current path isn't a valid workspace"""



from itsimodels.correlation_search import CorrelationSearch
from itsimodels.deep_dive import DeepDive
from itsimodels.entity_type import EntityType
from itsimodels.event_management_state import EventManagementState
from itsimodels.glass_table import GlassTable
from itsimodels.glass_table_icon import GlassTableIcon
from itsimodels.glass_table_image import GlassTableImage
from itsimodels.kpi_base_search import KpiBaseSearch
from itsimodels.kpi_threshold_template import KpiThresholdTemplate
from itsimodels.neap import NotableEventAggregationPolicy
from itsimodels.service import Service
from itsimodels.service_analyzer import ServiceAnalyzer
from itsimodels.service_template import ServiceTemplate
from itsimodels.team import Team


class ContentType(object):
    CORRELATION_SEARCH = 'correlation_searches'
    DEEP_DIVE = 'deep_dives'
    ENTITY_TYPE = 'entity_types'
    EVENT_MANAGEMENT_STATE = 'event_management_states'
    GLASS_TABLE = 'glass_tables'
    GLASS_TABLE_ICON = 'glass_table_icons'
    GLASS_TABLE_IMAGE = 'glass_table_images'
    KPI_BASE_SEARCH = 'kpi_base_searches'
    KPI_THRESHOLD_TEMPLATE = 'kpi_threshold_templates'
    NOTABLE_EVENT_AGGREGATION_POLICY = 'notable_event_aggregation_policies'
    SERVICE_ANALYZER = 'service_analyzers'
    SERVICE_TEMPLATE = 'service_templates'
    SERVICE = 'services'
    TEAM = 'teams'


ContentTypes = {
    ContentType.CORRELATION_SEARCH: CorrelationSearch,
    ContentType.DEEP_DIVE: DeepDive,
    ContentType.ENTITY_TYPE: EntityType,
    ContentType.EVENT_MANAGEMENT_STATE: EventManagementState,
    ContentType.GLASS_TABLE: GlassTable,
    ContentType.GLASS_TABLE_ICON: GlassTableIcon,
    ContentType.GLASS_TABLE_IMAGE: GlassTableImage,
    ContentType.KPI_BASE_SEARCH: KpiBaseSearch,
    ContentType.KPI_THRESHOLD_TEMPLATE: KpiThresholdTemplate,
    ContentType.NOTABLE_EVENT_AGGREGATION_POLICY: NotableEventAggregationPolicy,
    ContentType.SERVICE: Service,
    ContentType.SERVICE_ANALYZER: ServiceAnalyzer,
    ContentType.SERVICE_TEMPLATE: ServiceTemplate,
    ContentType.TEAM: Team
}


def get_content_type_for_model_class(model_class):
    for content_type, cls in ContentTypes.items():
        if cls == model_class:
            return content_type
    return None



from itsicli.setup_logging import logger
import re
import configparser

from pathlib import Path

from itsicli.content_packs.commands.base import BaseCommand, request_input
from itsicli.content_packs.files import ContentPackConfig
from itsicli.content_packs.scaffold import Scaffolder

INVALID_ID = 'Content pack id can only contain alphanumeric, underscore, or dash characters.\n'
CP_PREFIX = 'DA-ITSI-CP-'


def valid_id(id_value):
    return re.search('^[\w\-_]+$', id_value)


def load_conf(path):
    confparser = configparser.ConfigParser()
    confparser.read(path)
    return confparser


class InitCommand(BaseCommand):
    HELP = 'initialize an ITSI Content Pack'

    NAME = 'init'
    
    ITSI_DIR = 'itsi'
    DEFAULT_DIR = 'default'
    APPSERVER_DIR = 'appserver'

    def __init__(self, content_pack_id=None, content_pack_title=None, base_dir=Path.cwd()):
        super().__init__()
        self.content_pack_id = content_pack_id
        self.content_pack_title = content_pack_title
        self.base_dir = base_dir
        
        
    class Args(object):
        CONTENT_PACK_ID = 'content_pack_id'
        CONTENT_PACK_TITLE = 'content_pack_title'

    @classmethod
    def add_to_parser(cls, parser):
        subparser = parser.add_parser(cls.NAME, help=cls.HELP)
        subparser.add_argument('--{}'.format(cls.Args.CONTENT_PACK_ID), help='the content pack id')
        subparser.add_argument('--{}'.format(cls.Args.CONTENT_PACK_TITLE), help='the content pack title')

    def run(self, args):
        logger.info("Initializing content pack...")

        # files under itsi dir
        self.INPUT_ARGS = args

        # files under itsi dir
        logger.info('\nProcessing directory: {} -->'.format(self.ITSI_DIR))
        config = self.init_config(args)
        self.scaffold_itsi_files()

        logger.info('\nProcessing directory: {} -->'.format(self.DEFAULT_DIR))
        self.scaffold_default_files(config)

        logger.info('\nProcessing directory: {} -->'.format(self.APPSERVER_DIR))
        self.scaffold_appserver_files()

        logger.info("\nCompleted initializing content pack {}.".format(config.get_cp_id()))

    def scaffold_itsi_files(self):
        scaffolder = Scaffolder(self.base_dir.joinpath(self.ITSI_DIR))

        scaffolder.create_file('README.md')
        scaffolder.create_file('manifest.json')

    def scaffold_default_files(self, config):
        # app.conf
        scaffolder = Scaffolder(self.base_dir.joinpath(self.DEFAULT_DIR))
        app_conf_file = scaffolder.create_file('app.conf')
        self.update_app_conf_with_config(config, app_conf_file)

    def scaffold_appserver_files(self):
        appserver_dirs = self.base_dir.joinpath(self.APPSERVER_DIR).joinpath('static', 'screenshots')
        if appserver_dirs.exists():
            logger.info('{} already exists.'.format(appserver_dirs))
        else:
            appserver_dirs.mkdir(parents=True, exist_ok=True)
            logger.info('Created dirs {}'.format(appserver_dirs))

    def update_app_conf_with_config(self, config, app_conf_file):
        app_conf = load_conf(app_conf_file)
        app_conf['ui']['label'] = config.get_cp_title()
        app_conf['package']['id'] = config.get_cp_id()
        app_conf['launcher']['description'] = 'Description for ' + config.get_cp_title()

        with open(app_conf_file, 'w') as conf_file:
            app_conf.write(conf_file)
            logger.info('Completed updating {} with content pack config.'.format(app_conf_file))

    def load_cp_title_config(self, args, existing_config, new_data):
        existing_title = existing_config.get_cp_title()
        existing_title = existing_config.get_cp_title()
        content_pack_title = getattr(args, self.Args.CONTENT_PACK_TITLE)

        if not content_pack_title and not existing_title:
            new_data[ContentPackConfig.attr_title] = request_input('Content pack title')

        return new_data

    def load_cp_id_config(self, args, existing_config, new_data):
        existing_id = existing_config.get_cp_id()
        cp_id = getattr(args, self.Args.CONTENT_PACK_ID)

        if cp_id and not valid_id(cp_id):
            logger.error(INVALID_ID)

            new_data[ContentPackConfig.attr_id] = self.request_content_pack_id()
        elif not existing_id:
            new_data[ContentPackConfig.attr_id] = self.request_content_pack_id()

        return new_data

    def persist_new_config(self, config, new_data):
        if config.exists():
            logger.info('Updating file at {}'.format(config.path))
        else:
            config.path.parent.mkdir()
            logger.info('Creating file at {}'.format(config.path))

        config.update(new_data)
        config.write()

    # load existing config data
    # fill in from arg input or user input for missing data (id, title)
    # return the config
    def init_config(self, args):
        config = ContentPackConfig(self.base_dir)
        new_data = {}
        if self.content_pack_id:
            new_data[ContentPackConfig.attr_id] = self.content_pack_id
        else:
            new_data = self.load_cp_id_config(args, config, new_data)
        if self.content_pack_title:
            new_data[ContentPackConfig.attr_title] = self.content_pack_title
        else:
            new_data = self.load_cp_title_config(args, config, new_data)
        if not new_data:
            logger.info('Content pack {} already has config initialized.'.format(config.get_cp_id()))
            return config

        self.persist_new_config(config, new_data)

        return config


    def request_content_pack_id(self):
        while True:
            content_pack_id = input('Content pack id (will prefix with DA-ITSI-CP- if not provided) : ')

            if not valid_id(content_pack_id):
                logger.error(INVALID_ID)
                continue

            if content_pack_id:
                break

        if not content_pack_id.startswith(CP_PREFIX):
            content_pack_id = CP_PREFIX + content_pack_id
            logger.info('NOTE: Auto appended {} to content pack id --> {}'.format(
                CP_PREFIX,
                content_pack_id
            ))

        return content_pack_id


from pathlib import Path

from itsicli.content_packs.files import ContentPackConfig


def root_path():
    """
    Returns the best-guess root path for the content pack itsi dir, starting from the current working directory.

    :return: the root path for the given path
    :rtype: Workspace or None
    """
    curr = Path.cwd()

    while curr != curr.parent:
        config = curr.joinpath(ContentPackConfig.file_dir, ContentPackConfig.file_name)

        if config.exists():
            return curr

        curr = curr.parent

    return None



# ${copyright}

import importlib
import re

from itsicli.setup_logging import logger
from itsimodels.core.fields import (
    DictField,
    ForeignKey,
    ForeignKeyList,
    ForeignKeyListStr,
    ForeignKeyMultiEmbeddedStr,
    ListField,
    TypeField
)
from itsimodels.core.base_models import BaseModel
from itsimodels.team import GLOBAL_TEAM_KEY

FIELDS_TO_REMAP = {
    'action.itsi_event_generator.param.service_ids': 'itsimodels.service.Service',
    'action.itsi_event_generator.param.ad_at_kpi_ids': 'itsimodels.service.Kpi'
}


class KeysUpdater(object):

    def __init__(self, remapped_keys):
        self.remapped_keys = remapped_keys

    def update(self, model):
        for field_name, field in model.fields.items():
            field_value = getattr(model, field_name, None)
            if field_value is None:
                continue

            if isinstance(field, ForeignKeyListStr):
                self.update_foreign_key_list_str(model, field_name, field)

            elif field_name in FIELDS_TO_REMAP.keys():
                self.update_field_keys_list_str(model, field_name)

            elif isinstance(field, ForeignKeyMultiEmbeddedStr):
                self.update_foreign_key_multi_embedded_str(model, field_name, field)

            elif isinstance(field, ForeignKey):
                self.update_foreign_key(model, field_name, field)

            elif isinstance(field, ForeignKeyList):
                self.update_foreign_key_list(model, field_name, field)

            elif isinstance(field, ListField) and field.subtype and issubclass(field.subtype, BaseModel):
                self.update_list_of_models(field_value)

            elif isinstance(field, DictField) and field.subtype and issubclass(field.subtype, BaseModel):
                self.update_dict_of_models(field_value)

            elif isinstance(field, TypeField) and field.type and issubclass(field.type, BaseModel):
                self.update(field_value)

        return model

    def update_foreign_key(self, model, field_name, field):
        old_key = getattr(model, field_name)

        if not old_key or (field.refers and field.refers == 'itsimodels.team.Team' and
                           field_name == 'team_id' and old_key == GLOBAL_TEAM_KEY):
            return

        if field.key_regex:
            match = re.search(field.key_regex, old_key, flags=re.IGNORECASE)
            if not match:
                logger.warning(
                    'WARNING: Not able to find match for replacement in field_name={} with value={} using regex={} model={}'.format(
                        field_name, old_key, field.key_regex, model
                    ))
                return

            parsed_old_key = match.groups()[0]

            remapped_key = self.get_remapped_key(field.refers, parsed_old_key)

            new_key = old_key.replace(parsed_old_key, remapped_key)

        else:
            new_key = self.get_remapped_key(field.refers, old_key)

        if not new_key:
            logger.warning(
                'WARNING: Keep old key because not able to find any new key to map to old key={} for model={} field_name={}'.format(
                    old_key, model, field_name
                ))
        elif new_key == old_key:
            logger.warning(
                'WARNING: Old key={} is the same as new key for Service template object having it in field_name={}. '
                'Please verify that this key is valid and is not a dangling pointer.'.format(
                    old_key, field_name
                ))
        else:
            setattr(model, field_name, new_key)

    def update_foreign_key_list(self, model, field_name, field):
        foreign_key_list = getattr(model, field_name)

        if not foreign_key_list:
            return

        new_keys = []

        for old_key in foreign_key_list:
            new_key = self.get_remapped_key(field.refers, old_key)
            if new_key == old_key:
                logger.warning(
                    'WARNING: Old key={} is the same as new key for Service template object having it in field_name={}. '
                    'Please verify that this key is valid and is not a dangling pointer.'.format(
                        old_key, field_name
                    ))
            new_keys.append(new_key)

        setattr(model, field_name, new_keys)

    def get_unique_keys(self, matched_keys, model, field_name, old_key_str):
        unique_keys = set()
        if not matched_keys:
            return unique_keys

        # depending on the regex, if there are multiple groups
        # the matched_keys would be a list of tuples where each tuple contains
        # the same number of groups where some will be filled in if a match
        # was found. some can be empty ''
        # if there's only 1 matching group then it is a list of matched strings
        for item in matched_keys:
            if isinstance(item, str):
                if item: unique_keys.add(item)
            elif isinstance(item, tuple):
                unique_keys.update((match_key for match_key in item if match_key))
            else:
                logger.error('ERROR: Invalid matched_key type {}. Not able to process \
keys for old key str={} for model={} field_name={}'.format(
                    type(item), old_key_str, model, field_name)
                )

        return unique_keys

    def get_matched_old_key_to_new_key(self, model, field_name, field):
        old_key_str = getattr(model, field_name)

        old_to_new_key = {}
        if not old_key_str:
            return old_to_new_key

        match = re.findall(field.key_regex, old_key_str)
        if not match:
            logger.warning('WARNING: Not able to find any foreign keys to replace for old \
key str={} for model={} field_name={}'.format(
                old_key_str, model, field_name
            ))
            return old_to_new_key

        unique_old_keys = self.get_unique_keys(match, model, field_name, old_key_str)

        # logger.info('****** unique_old_keys={}'.format(unique_old_keys))
        valid_warnings = 0
        for parsed_old_key in unique_old_keys:

            # if the old key starts with da-itsi-cp most likely it was already
            # replace in extractor logic, otherwise, should check if something is
            # amiss
            if parsed_old_key.startswith('da-itsi-cp-'):
                continue

            remapped_key = self.get_remapped_key(field.refers, parsed_old_key)
            if remapped_key == parsed_old_key:
                logger.warning(
                    'WARNING: Old key={} is the same as new key for Service template object having it in field_name={}. '
                    'Please verify that this key is valid and is not a dangling pointer.'.format(
                        parsed_old_key, field_name
                    ))
            if not remapped_key or remapped_key == parsed_old_key:
                logger.warning(
                    'WARNING: Not able to find any new key to map to old key={} for model={} field_name={}'.format(
                        parsed_old_key, model, field_name
                    ))
                valid_warnings += 1
                if not remapped_key: remapped_key = parsed_old_key

            # we will keep the mapping even if the old and new are the same
            # this is useful for foreign key list str where we are replacing
            # the entire str
            old_to_new_key[parsed_old_key] = remapped_key

        if not old_to_new_key and valid_warnings > 0:
            logger.warning(
                'WARNING: Not able to find any new key to map to any old keys str={} for model={} field_name={}'.format(
                    old_key_str, model, field_name
                ))
        else:
            logger.info('Will remap these keys from old-->new {} for old keys str={} model={} field_name={}'.format(
                old_to_new_key, old_key_str, model, field_name))
        return old_to_new_key

    def update_foreign_key_list_str(self, model, field_name, field):

        old_to_new_key_map = self.get_matched_old_key_to_new_key(model, field_name, field)

        if not old_to_new_key_map:
            return

        # assuming this type of foreign key string is a list of keys
        # separated by , we only need the new keys
        new_key_list = []
        for new_key in old_to_new_key_map.values():
            new_key_list.append(new_key)

        setattr(model, field_name, ','.join(new_key_list))

    def update_foreign_key_multi_embedded_str(self, model, field_name, field):
        old_key_str = getattr(model, field_name)
        if not old_key_str:
            return

        old_to_new_key_map = self.get_matched_old_key_to_new_key(model, field_name, field)
        if not old_to_new_key_map:
            return

        for old_key, new_key in old_to_new_key_map.items():
            replaced_str = old_key_str.replace(old_key, new_key)
            old_key_str = replaced_str

        setattr(model, field_name, replaced_str)

    def update_list_of_models(self, child_models):
        for child_model in child_models:
            self.update(child_model)

    def update_dict_of_models(self, dict_models):
        for _, child_model in dict_models.items():
            self.update(child_model)

    def get_remapped_key(self, model_import_name, old_key):
        if model_import_name:
            parts = model_import_name.split('.')
            pkg_name = '.'.join(parts[:-1])
            model_name = parts[-1]

            try:
                pkg = importlib.import_module(pkg_name)
            except ImportError as exc:
                logger.exception('ERROR: not able to import module {}'.format(pkg_name))
                return old_key

            model_class = getattr(pkg, model_name)

            new_keys = self.remapped_keys.get(model_class, {})

            new_key = new_keys.get(old_key, old_key)

            return new_key
        else:
            # this foreign key has no 'refers', which means we don't know
            # what kind of object key this belongs to(i.e. url in glass table
            # eventHandlers(event_handlers)-->options-->url)
            # we will go through all objects' remapped keys
            for _, model_key_map_dict in self.remapped_keys.items():
                new_key = model_key_map_dict.get(old_key, None)

                if new_key:
                    return new_key

            return old_key

    # this method maps comma separated keys in a string to new key format
    def update_field_keys_list_str(self, model, field_name):
        old_key_ids = getattr(model, field_name)
        updated_key_str_list = ''
        for old_key_id in old_key_ids.split(','):
            new_key_id = self.get_remapped_key(FIELDS_TO_REMAP.get(field_name), old_key_id)
            if new_key_id == old_key_id:
                logger.warning(
                    'WARNING: Old key={} is the same as new key for Service template object having it in field_name={}. '
                    'Please verify that this key is valid and is not a dangling pointer.'.format(
                        old_key_id, field_name
                    ))
            if updated_key_str_list == '':
                updated_key_str_list = updated_key_str_list + new_key_id
            else:
                updated_key_str_list = updated_key_str_list + ',' + new_key_id
        setattr(model, field_name, updated_key_str_list)



import re
from itsicli.content_packs.model_formats import IMAGE_MIMETYPE_TO_EXT
from itsicli.setup_logging import logger

from copy import deepcopy
from slugify import slugify

from itsimodels.core.base_models import KEY_FIELD_NAME, ValidationError
from itsimodels.correlation_search import CorrelationSearch
from itsimodels.deep_dive import DeepDive
from itsimodels.entity_type import EntityType
from itsimodels.event_management_state import EventManagementState
from itsimodels.glass_table import GlassTable
from itsimodels.glass_table_icon import GlassTableIcon
from itsimodels.glass_table_image import GlassTableImage
from itsimodels.kpi_base_search import KpiBaseSearch, SearchMetric
from itsimodels.kpi_threshold_template import KpiThresholdTemplate
from itsimodels.neap import NotableEventAggregationPolicy
from itsimodels.service import Kpi, Service
from itsimodels.service_analyzer import ServiceAnalyzer
from itsimodels.service_template import ServiceTemplate
from itsimodels.team import Team, GLOBAL_TEAM_KEY
from itsicli.content_packs.backup.field_decode import BackupFieldDecoder


def extractor_registry():
    return {
        'base_service_template': (ServiceTemplate, Extractor),
        'correlation_search': (CorrelationSearch, ConfExtractor),
        'deep_dive': (DeepDive, Extractor),
        'entity_type': (EntityType, Extractor),
        'event_management_state': (EventManagementState, Extractor),
        'glass_table': (GlassTable, GlassTableExtractor),
        'glass_table_icons': (GlassTableIcon, GlassTableIconExtractor),
        'glass_table_images': (GlassTableImage, GlassTableImageExtractor),
        'home_view': (ServiceAnalyzer, ServiceAnalyzerExtractor),
        'kpi_base_search': (KpiBaseSearch, KpiBaseSearchExtractor),
        'kpi_threshold_template': (KpiThresholdTemplate, Extractor),
        'notable_aggregation_policy': (NotableEventAggregationPolicy, Extractor),
        'service': (Service, ServiceExtractor),
        'team': (Team, TeamExtractor)
    }


class Extractor(object):

    def __init__(self, model_class, remapped_keys, prefix=''):
        self.model_class = model_class
        self.remapped_keys = remapped_keys
        self.prefix = prefix

    def extract(self, raw_data):
        models = []

        for content in raw_data:
            model_data = self.extract_model_data(content)
            if not model_data:
                continue

            try:
                logger.debug('Validating model_data="{}"'.format(model_data))

                model = self.model_class(model_data, field_decoder=BackupFieldDecoder())
            except ValidationError as exc:
                logger.warning('WARNING: Failed to extract {}'.format(self.model_class))
                logger.exception(exc)
            else:
                models.append(model)

        return models

    def extract_model_data(self, raw_data):
        immutable = str(raw_data.get('_immutable', 0))
        if immutable == '1' or immutable == 'True':
            return None

        object_key = self.get_object_key(raw_data)
        if not object_key:
            return None

        object_title = self.get_object_title(raw_data)
        if not object_title:
            return None

        old_object_key, new_object_key = self.apply_prefix(raw_data)

        self.remap_keys(old_object_key, new_object_key)

        raw_data[self.key_field] = new_object_key

        return raw_data

    def apply_prefix(self, raw_data):
        old_object_key = raw_data.get(self.raw_key_field)

        object_title = self.get_object_title(raw_data)

        new_object_key = '{}{}'.format(self.prefix, slugify(object_title))

        return old_object_key, new_object_key

    def remap_keys(self, old_object_key, new_object_key):
        if not old_object_key:
            return

        remapped_keys = self.remapped_keys.setdefault(self.model_class, {})
        remapped_keys[old_object_key] = new_object_key

    def get_object_key(self, raw_data):
        return raw_data.get(self.raw_key_field)

    def get_object_title(self, raw_data):
        return raw_data.get('title', '')

    def get_object_name(self, raw_data):
        return raw_data.get('name', '')

    @property
    def raw_key_field(self):
        key_field = getattr(self.model_class, KEY_FIELD_NAME)
        return key_field.alias

    @property
    def key_field(self):
        return self.raw_key_field


class ConfExtractor(Extractor):

    def extract_model_data(self, raw_data):
        content = super(ConfExtractor, self).extract_model_data(raw_data)

        data = deepcopy(content)
        data.pop('object_type', None)

        return data

    def get_object_title(self, raw_data):
        return self.get_object_key(raw_data)

    @property
    def raw_key_field(self):
        return 'name'

    @property
    def key_field(self):
        return KEY_FIELD_NAME


class ServiceExtractor(Extractor):

    SERVICE_HEALTH_SEARCH_RE = r'`get_full_itsi_summary_service_health_events\((.+)\)`'

    def extract_model_data(self, raw_data):
        old_service_key = raw_data[self.raw_key_field]

        model_data = super(ServiceExtractor, self).extract_model_data(raw_data)

        service_key = model_data[self.raw_key_field]

        kpi_remapped_keys = self.remapped_keys.setdefault(Kpi, {})

        kpis = raw_data.get('kpis', [])

        for kpi in kpis:
            old_kpi_key = kpi.get('_key', '')

            if kpi.get('type') == 'service_health':
                new_kpi_key = 'SHKPI-{}'.format(service_key)

                if old_kpi_key == new_kpi_key:
                    continue # already in new format

                base_search = kpi.get('base_search', '')
                kpi['base_search'] = base_search.replace(old_service_key, service_key)

                search = kpi.get('search', '')
                kpi['search'] = search.replace(old_service_key, service_key)
            else:
                if old_kpi_key.startswith(self.prefix):
                    continue # already in new format

                new_kpi_key = '{}{}'.format(self.prefix, old_kpi_key)

            kpi['_key'] = new_kpi_key
            kpi_remapped_keys[old_kpi_key] = new_kpi_key

        service_tags = raw_data.get('service_tags', {})
        if isinstance(service_tags, list) and not service_tags:
            # The default value of service tags is unfortunately an empty list, but
            # the model definition expects a dictionary in order to pass model validation
            raw_data['service_tags'] = {}

        return model_data


class TeamExtractor(Extractor):

    def extract_model_data(self, raw_data):
        if raw_data[self.raw_key_field] == GLOBAL_TEAM_KEY:
            return None

        return super(TeamExtractor, self).extract_model_data(raw_data)


class GlassTableExtractor(Extractor):

    def extract_model_data(self, raw_data):
        definition = raw_data.get('definition')
        if not definition:
            return None

        self.fix_series_colors(definition)

        return super(GlassTableExtractor, self).extract_model_data(raw_data)

    def fix_series_colors(self, definition):
        visualizations = definition['visualizations']
        if not visualizations:
            return

        for key, value in visualizations.items():
            if value['type'] == 'viz.area' and value['options']:
                series_colors = value['options']['seriesColors']
                if not series_colors or isinstance(series_colors, list):
                    continue

                if not isinstance(series_colors, str):
                    continue

                # observed there's cases where instead of a list
                # backup contains the value a string: '[#FFFFFF]'
                # this causes problem when we are validating it as
                # a list field, thus converting to list if applicable
                series_colors = re.sub('[\[\]]', '', series_colors)
                series_colors_list = re.split(',', series_colors)
                value['options']['seriesColors'] = series_colors_list


class GlassTableImageExtractor(Extractor):

    def extract_model_data(self, raw_data):
        object_key = self.get_object_key(raw_data)
        if not object_key:
            return None

        old_object_key, new_object_key = self.apply_prefix(raw_data)

        self.remap_keys(old_object_key, new_object_key)

        raw_data[self.raw_key_field] = new_object_key

        return raw_data


    def get_img_name_without_suffix(self, raw_data):
        object_name = self.get_object_name(raw_data)

        type = raw_data.get('type')
        if not type:
            return object_name

        suffix = IMAGE_MIMETYPE_TO_EXT[type]
        if suffix and object_name.endswith(suffix):
            return object_name[:-len(suffix)]

        return object_name


    def apply_prefix(self, raw_data):
        old_object_key = raw_data.get(self.raw_key_field)

        object_name = self.get_img_name_without_suffix(raw_data)

        if object_name.startswith(self.prefix):
            new_object_key = slugify(object_name)
        else:
            new_object_key = '{}{}'.format(self.prefix, slugify(object_name))

        return old_object_key, new_object_key


class GlassTableIconExtractor(GlassTableImageExtractor):

    def apply_prefix(self, raw_data):
        old_object_key = raw_data.get(self.raw_key_field)

        object_title = self.get_object_title(raw_data)

        new_object_key = '{}{}'.format(self.prefix, slugify(object_title))

        return old_object_key, new_object_key


class KpiBaseSearchExtractor(Extractor):

    def extract_model_data(self, raw_data):
        model_data = super(KpiBaseSearchExtractor, self).extract_model_data(raw_data)
        if not model_data:
            return None

        metric_remapped_keys = self.remapped_keys.setdefault(SearchMetric, {})

        metrics = raw_data.get('metrics', [])

        for metric in metrics:
            old_metric_key = metric.get(self.raw_key_field)
            new_metric_key = slugify(metric.get('title', ''))

            metric[self.raw_key_field] = new_metric_key
            metric_remapped_keys[old_metric_key] = new_metric_key

        return model_data


class ServiceAnalyzerExtractor(Extractor):

    def extract_model_data(self, raw_data):
        # default service analyzer record should be skipped. default
        # is generated on per user basis
        if raw_data['isDefault']:
            return None

        return super(ServiceAnalyzerExtractor, self).extract_model_data(raw_data)




# ${copyright}

from itsimodels.core.field_decode import FieldDecoder


class BackupFieldDecoder(FieldDecoder):
    """Decodes data from an ITSI backup object into the appropriate model format"""

    def decode_field_name(self, field, field_name):
        return field.alias or field_name



# ${copyright}

from itsicli.setup_logging import logger
from pathlib import Path


HERE = Path(__file__).resolve().parent

RESOURCES = HERE.joinpath('resources')


class Scaffolder(object):

    def __init__(self, root):
        self.root = root

    def create_file(self, filename):
        path = self.root.joinpath(filename)
        if path.exists():
            logger.info('{} already exists.'.format(path))
            return path

        template_path = RESOURCES.joinpath(filename)
        if template_path.is_dir():
            logger.info('Creating directory at {} ...'.format(path))
            path.mkdir()
        else:
            logger.info('Creating file at {} ...'.format(path))
            contents = template_path.read_bytes()
            path.parent.mkdir(parents=True, exist_ok=True)
            path.write_bytes(contents)

        return path



from itsicli.content_packs.workspace import root_path
from itsicli.content_packs.validate.result import Level, result


class ValidateMetadata(object):

    in_progress_text = 'Checking for metadata'
    meta_data_dir = ['metadata']

    def run(self, *args, **kwargs):
        path = root_path()
        metadata_dir_path = path.joinpath(*self.meta_data_dir)
        matadata_default_conf_file = path.joinpath(
            *self.meta_data_dir, 'default.meta')

        if metadata_dir_path.exists():
            results = [
                result(Level.WARNING, "metadata directory found at '{}', please remove this directory or reach out to Content Library team in case of concerns.".format(
                    metadata_dir_path))
            ]
            if matadata_default_conf_file.exists():
                results.append(
                    result(Level.WARNING, "default.meta file found at '{}', please remove this file or reach out to Content Library team in case of concerns.".format(
                        matadata_default_conf_file))
                )
        else:
            results = []

        return results



# ${copyright}

import json
import os

from collections import defaultdict

from itsicli.content_packs.backup.extract_models import extractor_registry
from itsicli.content_packs.backup.remap_keys import KeysUpdater
from itsicli.content_packs.content_types import get_content_type_for_model_class


BACKUP_DELIM = '___'


def get_backup_object_type(file_path):
    if not os.path.isfile(file_path):
        return None

    _, ext = os.path.splitext(file_path)
    if ext != '.json':
        return None

    filename = os.path.basename(file_path)
    parts = filename.split(BACKUP_DELIM)
    if len(parts) < 2:
        return None

    object_type = parts[1]

    return object_type


def read_backup_file(file_path):
    with open(file_path, 'r') as fobj:
        raw_data = fobj.read()

        try:
            data = json.loads(raw_data)
        except ValueError:
            data = []

    return data


class BackupConverter(object):

    def __init__(self, prefix=''):
        self.prefix = prefix
        self.remapped_keys = {}

    def to_models(self, backup_dir):
        extracted_models = self.extract_models(backup_dir)

        updated_models = self.update_model_keys(extracted_models)

        return updated_models

    def extract_models(self, backup_dir):
        content_models = defaultdict(list)

        for root, _, files in os.walk(backup_dir):
            backup_files = [os.path.join(root, file_name) for file_name in files]

            for backup_file in backup_files:
                object_type = get_backup_object_type(backup_file)

                record = extractor_registry().get(object_type)
                if not record:
                    continue

                model_class, extractor_class = record

                raw_json = read_backup_file(backup_file)

                extractor = extractor_class(model_class, self.remapped_keys, self.prefix)
                models = extractor.extract(raw_json)

                content_type = get_content_type_for_model_class(extractor.model_class)
                content_models[content_type].extend(models)

            break

        return content_models

    def update_model_keys(self, extracted_models):
        keys_updater = KeysUpdater(self.remapped_keys)

        for content_type, models in extracted_models.items():
            for model in models:
                keys_updater.update(model)

        return extracted_models



import re
from itsicli.content_packs.files import ContentPackConfig
from itsicli.content_packs.validate.result import Level, result
from itsicli.content_packs.workspace import root_path


class ValidateConfig(object):

    in_progress_text = 'Checking {}'.format(ContentPackConfig.file_name)

    def run(self, *args, **kwargs):
        config = ContentPackConfig(root_path())

        results = []

        if not config.exists():
            results.append(
                result(
                    Level.ERROR, "Missing '{}' file.".format(config.path)
                )
            )
            return results

        title = config.title
        if not title:
            results.append(
                result(
                    Level.ERROR, "Add a 'title' setting in '{}'.".format(config.path)
                )
            )

        version = config.version
        if not version:
            results.append(
                result(
                    Level.ERROR, "Add a valid 'version' setting in '{}'.".format(config.path)
                )
            )
        else:
            version_re = r'^(\d+\.)(\d+\.)(\*|\d+)$'
            matched = re.match(version_re, version)
            if not matched:
                results.append(
                    result(
                        Level.ERROR, "'version' {} in '{}' does not conform to x.x.x format.".format(
                            version, config.path)
                    ))

        description = config.description
        if not description:
            results.append(
                result(
                    Level.WARNING, "Add a valid 'description' setting in '{}'.".format(config.path)
                )
            )

        return results



from itsicli.content_packs.files import ContentPackConfig, ContentPackManifest, InvalidWorkspace
from itsicli.content_packs.workspace import root_path
from itsicli.setup_logging import logger

def request_input(prompt, validate=lambda x: x):
    while True:
        value = input('{}: '.format(prompt))
        if validate(value):
            break
    return value


class BaseCommand(object):

    @classmethod
    def add_to_parser(cls, parser):
        pass

    def run(self, args):
        raise NotImplementedError


class WorkspaceCommand(BaseCommand):

    def __init__(self):
        super().__init__()

        self.home_path = root_path()

        if not self.home_path:
            msg = '\n'.join([
                    "\nUnable to determine a valid workspace from your current path.",
                    "\nMake sure you're in the correct directory, or try running the 'init' command first."
                ])
            logger.error(msg)
            raise InvalidWorkspace(msg)

        self.config = ContentPackConfig(self.home_path)

        if not self.config.id:
            msg = '\n'.join([
                    '\nUnable to determine the current content pack id.'
                    "\nMake sure you're in the correct directory, or try running the 'init' command first."
                ])
            logger.error(msg)
            raise InvalidWorkspace(msg)


        self.manifest = ContentPackManifest(self.home_path)



from itsicli.setup_logging import logger
from itsicli.content_packs.commands.base import WorkspaceCommand
from itsicli.content_packs.validate.registry import VALIDATORS
from itsicli.content_packs.validate.result import Level, result


class ValidateCommand(WorkspaceCommand):

    HELP = 'validates whether a content pack is in a proper state'

    NAME = 'validate'

    @classmethod
    def add_to_parser(cls, parser):
        subparser = parser.add_parser(cls.NAME, help=cls.HELP)
        return subparser

    def run(self, args):
        num_passed = 0
        num_failed = 0

        for validator_class in VALIDATORS:
            validator = validator_class()

            in_progress_text = "\n{}...".format(validator.in_progress_text)
            logger.info(in_progress_text)

            try:
                results = validator.run(
                    home_path=self.home_path
                )
            except Exception as exc:
                logger.exception(exc)

                results = [
                    result(Level.ERROR, 'Yikes, an error occurred. See the above stack trace.')
                ]

            passed = self.print_results(results)
            if passed:
                num_passed += 1
            else:
                num_failed += 1

        logger.info('\nDone with {} passed, and {} failures.'.format(num_passed, num_failed))

    def print_results(self, results):
        passed = True

        for res in results:
            log = self.get_logger_func(res.level)
            log('\t[{}] {}'.format(res.level, res.message))

            if res.level == Level.ERROR:
                passed = False

        return passed

    def get_logger_func(self, level):
        if level == Level.WARNING:
            return logger.warning

        if level == Level.ERROR:
            return logger.error

        return logger.info



from itsicli.content_packs.files import CONTENT_PACK_SCREENSHOTS_DIR, ContentPackManifest
from itsicli.content_packs.validate.result import Level, result


class ValidateScreenshots(object):

    in_progress_text = 'Checking for screenshots'

    def run(self, home_path, *args, **kwargs):
        results = []

        manifest = ContentPackManifest(home_path)

        if not manifest.path.exists():
            results.append(
                result(
                    Level.ERROR, "Missing manifest '{}' file.".format(manifest.path)
                )
            )
            return results

        if not manifest.main_screenshot:
            results.append(
                result(
                    Level.ERROR, "Define a '{}' property in '{}'.".format(manifest.attr_main_screenshot, manifest.path)
                )
            )
        else:
            results.extend(
                self.validate_screenshot(manifest.main_screenshot,
                                         home_path=home_path,
                                         name=manifest.attr_main_screenshot)
            )

        screenshots = manifest.screenshots or []
        if not screenshots:
            results.append(
                result(
                    Level.WARNING, "Try to include some screenshots with the '{}' property in '{}'.".format(
                        manifest.screenshots, manifest.path)
                )
            )
        else:
            for index, screenshot in enumerate(screenshots):
                name = 'screenshot {}'.format(index + 1)
                results.extend(
                    self.validate_screenshot(screenshot, home_path=home_path, name=name)
                )

        return results

    def validate_screenshot(self, screenshot, home_path, name):
        results = []

        def validate_screenshot_attr(attr):
            file_name = screenshot.get(attr)

            if not file_name:
                results.append(
                    result(Level.ERROR, "Add a valid '{}' property for '{}'".format(attr, name))
                )

            else:
                screenshots_dir = home_path.joinpath(*CONTENT_PACK_SCREENSHOTS_DIR)

                image_path = screenshots_dir.joinpath(file_name)

                if not image_path.exists() or image_path.is_dir():
                    results.append(
                        result(Level.ERROR, "Add a valid screenshot image at '{}'".format(image_path))
                    )

        validate_screenshot_attr('thumb')
        validate_screenshot_attr('path')

        return results



# ${copyright}

import os

from pathlib import Path

from itsicli.setup_logging import logger
from itsicli.content_packs.backup.workspace_import import BackupImporter
from itsicli.content_packs.commands.base import WorkspaceCommand


class ImportBackupCommand(WorkspaceCommand):

    HELP = 'import the content in a backup file'

    NAME = 'importbackup'

    class Args(object):
        BACKUP_FILE_PATH = 'backup_file_path'

    @classmethod
    def add_to_parser(cls, parser):
        subparser = parser.add_parser(cls.NAME, help=cls.HELP)
        subparser.add_argument('{}'.format(cls.Args.BACKUP_FILE_PATH), help='the ITSI backup file path')
        return subparser

    def run(self, args):
        backup_file = getattr(args, self.Args.BACKUP_FILE_PATH)

        logger.info("Backup file path is set to '{}'".format(backup_file))

        self.import_from_backup_file(self.config.id, backup_file)

        logger.info('Done.')

    def import_from_backup_file(self, content_pack_id, backup_file):
        if not backup_file:
            return

        backup_path = Path(os.path.expanduser(backup_file)).resolve()

        importer = BackupImporter(content_pack_id)
        importer.import_backup(backup_path)



import base64
import configparser
import io

from itsimodels.core.base_models import KEY_FIELD_NAME


def to_conf(model):
    config = configparser.ConfigParser(
        allow_no_value=True,
        interpolation=None
    )
    config.optionxform = str

    key = model.get_key()

    model_data = model.to_dict()

    # Remove the key fields from the data before converting into conf format
    model_data.pop(KEY_FIELD_NAME, None)

    key_field = getattr(model.__class__, KEY_FIELD_NAME)
    if key_field and key_field.alias:
        model_data.pop(key_field.alias, None)

    config_data = {
        key: model_data
    }
    config.read_dict(config_data)

    fobj = io.StringIO()
    config.write(fobj)
    fobj.seek(0)

    return fobj.read()


IMAGE_MIMETYPE_TO_EXT = {
    'image/x-ms-bmp': 'bmp',
    'image/gif': 'gif',
    'image/vnd.microsoft.icon': 'ico',
    'image/ief': 'ief',
    'image/jpeg': 'jpeg',
    'image/x-portable-bitmap': 'pbm',
    'image/x-portable-graymap': 'pgm',
    'image/png': 'png',
    'image/x-portable-anymap': 'pnm',
    'image/x-portable-pixmap': 'ppm',
    'image/x-cmu-raster': 'ras',
    'image/x-rgb': 'rgb',
    'image/svg+xml': 'svg',
    'image/tiff': 'tiff',
    'image/x-xbitmap': 'xbm',
    'image/x-xpixmap': 'xpm',
    'image/x-xwindowdump': 'xwd'
}


def to_image(model):
    data_parts = model.data.split(',')
    if not data_parts:
        return ''

    data_str = data_parts[0] if len(data_parts) == 1 else data_parts[1]
    data = base64.b64decode(data_str)

    return data


def image_file_extension(model):
    return IMAGE_MIMETYPE_TO_EXT[model.type]



import shutil

from pathlib import Path
from tempfile import TemporaryDirectory
from zipfile import ZipFile

from itsicli.content_packs.backup.convert_backup import BackupConverter
from itsicli.content_packs.workspace import root_path
from itsicli.content_packs.files import ContentPackManifest, ContentModel
from itsicli.setup_logging import logger

BACKUP_DIR = 'backup'
ITSI_DIR = 'itsi'


def unarchive(source_zip, dest_dir):
    if source_zip.is_dir():
        backup_dir = Path(dest_dir).joinpath(BACKUP_DIR)
        shutil.copytree(source_zip, backup_dir)
        return backup_dir

    zip_file = ZipFile(source_zip)
    zip_file.extractall(path=dest_dir)

    backup_dir = Path(dest_dir).joinpath(BACKUP_DIR)

    return backup_dir if backup_dir.is_dir() else None


class BackupImporter(object):
    def __init__(self, content_pack_id):
        self.content_pack_id = content_pack_id

    def import_backup(self, backup_path):
        with TemporaryDirectory() as extract_dir:
            backup_dir = unarchive(backup_path, extract_dir)
            if not backup_dir:
                msg = 'No {}/ directory found in the backup file'.format(BACKUP_DIR)
                logger.error(msg)
                raise Exception(msg)

            prefix = '{}-'.format(self.content_pack_id).lower()
            converter = BackupConverter(prefix)
            models = converter.to_models(backup_dir)

        self.write_models(models)

    def write_models(self, content_models):
        home_path = root_path()

        manifest = ContentPackManifest(home_path)

        for content_type, models in content_models.items():
            for model in models:
                try:
                    content_model = ContentModel(home_path, model)

                    if content_model.path.exists():
                        logger.info('Updating {}'.format(content_model.path))
                    else:
                        logger.info('Creating {}'.format(content_model.path))

                    content_model.write()
                except Exception as exc:
                    logger.error('Failed to create "{}" with id="{}"'.format(content_type, model.get_key()))
                    logger.exception(exc)
                else:
                    manifest.add_content_model(content_model)

        if manifest.path.exists():
            logger.info('Updating manifest file at {}'.format(manifest.path))
        else:
            logger.info('Creating manifest file at {}'.format(manifest.path))

        manifest.write()



import json

from itsicli.content_packs.commands.base import WorkspaceCommand
from itsicli.content_packs.content_types import ContentType, ContentTypes
from itsicli.content_packs.files import ITSI_DIR
from itsicli.setup_logging import logger

class TidyUpCommand(WorkspaceCommand):

    HELP = 'tidy up a content pack by removing extraneous files'

    NAME = 'tidyup'

    @classmethod
    def add_to_parser(cls, parser):
        subparser = parser.add_parser(cls.NAME, help=cls.HELP)
        return subparser

    def run(self, args):
        update_manifest = self.tidy_glass_tables()
        update_manifest = self.tidy_service_analyzers() or update_manifest

        if update_manifest:
            logger.info('Updating "{}"'.format(self.manifest.path))
            self.manifest.write()

    def tidy_glass_tables(self):
        glass_tables = getattr(self.manifest, ContentType.GLASS_TABLE, []) or []
        GlassTable = ContentTypes[ContentType.GLASS_TABLE]

        keep_glass_tables = []

        for glass_table in glass_tables:
            contents = self.read_contents(ContentType.GLASS_TABLE, glass_table)
            if not contents:
                continue

            model = GlassTable(contents)

            if not model.definition:
                logger.info('Removing definition-less glass table: "{}"'.format(model.title))
                file_path = self.content_file_path(ContentType.GLASS_TABLE, glass_table)
                file_path.unlink()
            else:
                keep_glass_tables.append(glass_table)

        if len(keep_glass_tables) != len(glass_tables):
            self.manifest.data[ContentType.GLASS_TABLE] = keep_glass_tables
            return True

        return False

    def tidy_service_analyzers(self):
        service_analyzers = getattr(self.manifest, ContentType.SERVICE_ANALYZER, []) or []
        ServiceAnalyzer = ContentTypes[ContentType.SERVICE_ANALYZER]

        keep_service_analyzers = []

        for service_analyzer in service_analyzers:
            contents = self.read_contents(ContentType.SERVICE_ANALYZER, service_analyzer)
            if not contents:
                continue

            model = ServiceAnalyzer(contents)

            if model.title == 'Service Analyzer':
                logger.info('Removing "{}"...'.format(model.title))
                file_path = self.content_file_path(ContentType.SERVICE_ANALYZER, service_analyzer)
                file_path.unlink()
            else:
                keep_service_analyzers.append(service_analyzer)

        if len(keep_service_analyzers) != len(service_analyzers):
            self.manifest.data[ContentType.SERVICE_ANALYZER] = keep_service_analyzers
            return True

        return False

    def read_contents(self, content_type, content_file):
        file_path = self.content_file_path(content_type, content_file)
        if not file_path.exists():
            return None

        with open(file_path, 'rb') as fobj:
            contents = json.loads(fobj.read())

        return contents

    def content_file_path(self, content_type, content_file):
        return self.home_path.joinpath(ITSI_DIR, content_type, content_file)




from itsicli.content_packs.validate.validate_config import ValidateConfig
from itsicli.content_packs.validate.validate_icon import ValidateIcon
from itsicli.content_packs.validate.validate_metadata import ValidateMetadata
from itsicli.content_packs.validate.validate_screenshots import ValidateScreenshots

VALIDATORS = [
    ValidateConfig,
    ValidateIcon,
    ValidateMetadata,
    ValidateScreenshots
]



from enum import Enum


def result(level, message):
    return ValidationResult(level, message)


class Level(Enum):
    INFO = 1
    WARNING = 2
    ERROR = 3

    def __str__(self):
        return str(self.name).lower()


class ValidationResult(object):

    def __init__(self, level, message):
        self.level = level
        self.message = message



import setuptools

from itsicli import __version__


with open('README.md', 'r') as fh:
    long_description = fh.read()


with open('requirements.txt') as fh:
    requirements = [line.strip() for line in fh.readlines()
                    if line.strip() and not line.startswith('--')]

setuptools.setup(
    name='itsicli',
    version=__version__,
    author='Splunk, Inc.',
    description='The ITSI Command Line Interface (CLI)',
    long_description=long_description,
    long_description_content_type='text/markdown',
    scripts=[
        'bin/itsi-content-pack'
    ],
    url='https://github.com/splunk/itsi-cli',
    packages=setuptools.find_packages(),
    package_data={
        'itsicli.content_packs': ['resources/*']
    },
    install_requires=requirements,
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)



from itsicli.content_packs.workspace import root_path
from itsicli.content_packs.validate.result import Level, result


class ValidateIcon(object):

    in_progress_text = 'Checking for icon'
    icon_dir = ('appserver', 'static')

    def run(self, *args, **kwargs):
        path = root_path()
        icon_path = path.joinpath(*self.icon_dir, 'icon.png')

        if not icon_path.exists():
            results = [
                result(Level.ERROR, "Add a 200x200px icon file at '{}'.".format(icon_path))
            ]
        else:
            results = []

        return results
